@@ -5,78 +5,72 @@
  *	on MacOS X.
  *
  * Copyright 2001, Apple Computer, Inc.
- * Copyright (c) 2005 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright (c) 2005-2007 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- *      The following terms apply to all files originating from Apple
- *      Computer, Inc. ("Apple") and associated with the software
- *      unless explicitly disclaimed in individual files.
+ *	The following terms apply to all files originating from Apple
+ *	Computer, Inc. ("Apple") and associated with the software
+ *	unless explicitly disclaimed in individual files.
  *
  *
- *      Apple hereby grants permission to use, copy, modify,
- *      distribute, and license this software and its documentation
- *      for any purpose, provided that existing copyright notices are
- *      retained in all copies and that this notice is included
- *      verbatim in any distributions. No written agreement, license,
- *      or royalty fee is required for any of the authorized
- *      uses. Modifications to this software may be copyrighted by
- *      their authors and need not follow the licensing terms
- *      described here, provided that the new terms are clearly
- *      indicated on the first page of each file where they apply.
+ *	Apple hereby grants permission to use, copy, modify,
+ *	distribute, and license this software and its documentation
+ *	for any purpose, provided that existing copyright notices are
+ *	retained in all copies and that this notice is included
+ *	verbatim in any distributions. No written agreement, license,
+ *	or royalty fee is required for any of the authorized
+ *	uses. Modifications to this software may be copyrighted by
+ *	their authors and need not follow the licensing terms
+ *	described here, provided that the new terms are clearly
+ *	indicated on the first page of each file where they apply.
  *
  *
- *      IN NO EVENT SHALL APPLE, THE AUTHORS OR DISTRIBUTORS OF THE
- *      SOFTWARE BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
- *      INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
- *      THIS SOFTWARE, ITS DOCUMENTATION, OR ANY DERIVATIVES THEREOF,
- *      EVEN IF APPLE OR THE AUTHORS HAVE BEEN ADVISED OF THE
- *      POSSIBILITY OF SUCH DAMAGE.  APPLE, THE AUTHORS AND
- *      DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
- *      BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
- *      FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS
- *      SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND APPLE,THE
- *      AUTHORS AND DISTRIBUTORS HAVE NO OBLIGATION TO PROVIDE
- *      MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
+ *	IN NO EVENT SHALL APPLE, THE AUTHORS OR DISTRIBUTORS OF THE
+ *	SOFTWARE BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
+ *	INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
+ *	THIS SOFTWARE, ITS DOCUMENTATION, OR ANY DERIVATIVES THEREOF,
+ *	EVEN IF APPLE OR THE AUTHORS HAVE BEEN ADVISED OF THE
+ *	POSSIBILITY OF SUCH DAMAGE.  APPLE, THE AUTHORS AND
+ *	DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
+ *	BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
+ *	FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.	 THIS
+ *	SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND APPLE,THE
+ *	AUTHORS AND DISTRIBUTORS HAVE NO OBLIGATION TO PROVIDE
+ *	MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
  *
- *      GOVERNMENT USE: If you are acquiring this software on behalf
- *      of the U.S. government, the Government shall have only
- *      "Restricted Rights" in the software and related documentation
- *      as defined in the Federal Acquisition Regulations (FARs) in
- *      Clause 52.227.19 (c) (2).  If you are acquiring the software
- *      on behalf of the Department of Defense, the software shall be
- *      classified as "Commercial Computer Software" and the
- *      Government shall have only "Restricted Rights" as defined in
- *      Clause 252.227-7013 (c) (1) of DFARs.  Notwithstanding the
- *      foregoing, the authors grant the U.S. Government and others
- *      acting in its behalf permission to use and distribute the
- *      software in accordance with the terms specified in this
- *      license.
+ *	GOVERNMENT USE: If you are acquiring this software on behalf
+ *	of the U.S. government, the Government shall have only
+ *	"Restricted Rights" in the software and related documentation
+ *	as defined in the Federal Acquisition Regulations (FARs) in
+ *	Clause 52.227.19 (c) (2).  If you are acquiring the software
+ *	on behalf of the Department of Defense, the software shall be
+ *	classified as "Commercial Computer Software" and the
+ *	Government shall have only "Restricted Rights" as defined in
+ *	Clause 252.227-7013 (c) (1) of DFARs.  Notwithstanding the
+ *	foregoing, the authors grant the U.S. Government and others
+ *	acting in its behalf permission to use and distribute the
+ *	software in accordance with the terms specified in this
+ *	license.
  *
- * RCS: @(#) $Id: tkMacOSXMouseEvent.c,v 1.6.2.13 2006/04/11 05:42:11 das Exp $
+ * RCS: @(#) $Id: tkMacOSXMouseEvent.c,v 1.6.2.19 2007/05/09 12:57:46 das Exp $
  */
 
 #include "tkMacOSXInt.h"
+#include "tkMacOSXWm.h"
 #include "tkMacOSXEvent.h"
 #include "tkMacOSXDebug.h"
 
-#if !defined(MAC_OS_X_VERSION_10_3) || \
-        (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_3)
-    /* Define constants only available on Mac OS X 10.3 or later */
-    #define kEventParamWindowPartCode 'wpar'
-    #define typeWindowPartCode        'wpar'
-#endif
-
 typedef struct {
     WindowRef	   whichWin;
     WindowRef	   activeNonFloating;
     WindowPartCode windowPart;
-    Point	   global;
-    Point	   local;
     unsigned int   state;
     long	   delta;
     Window	   window;
+    Point	   global;
+    Point	   local;
 } MouseEventData;
 
 /*
@@ -97,41 +91,42 @@ static int GenerateToolbarButtonEvent(MouseEventData * medPtr);
 static int HandleWindowTitlebarMouseDown(MouseEventData * medPtr, Tk_Window tkwin);
 static unsigned int ButtonModifiers2State(UInt32 buttonState, UInt32 keyModifiers);
 
-static int TkMacOSXGetEatButtonUp();
+static int TkMacOSXGetEatButtonUp(void);
 static void TkMacOSXSetEatButtonUp(int f);
 
+
 /*
  *----------------------------------------------------------------------
  *
  * TkMacOSXProcessMouseEvent --
  *
- *	  This routine processes the event in eventPtr, and
- *	  generates the appropriate Tk events from it.
+ *	This routine processes the event in eventPtr, and
+ *	generates the appropriate Tk events from it.
  *
  * Results:
- *	  True if event(s) are generated - false otherwise.
+ *	True if event(s) are generated - false otherwise.
  *
  * Side effects:
- *	  Additional events may be place on the Tk event queue.
+ *	Additional events may be place on the Tk event queue.
  *
  *----------------------------------------------------------------------
  */
 
-int
+MODULE_SCOPE int
 TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 {
-    Tk_Window	   tkwin;
-    Point	   where, where2;
-    int		   xOffset, yOffset, result;
-    TkDisplay *	   dispPtr;
-    OSStatus	   status;
+    Tk_Window tkwin;
+    Point where, where2;
+    int result;
+    TkDisplay * dispPtr;
+    OSStatus err;
     MouseEventData mouseEventData, * medPtr = &mouseEventData;
     ProcessSerialNumber frontPsn, ourPsn = {0, kCurrentProcess};
-    Boolean	   isFrontProcess = true;
+    Boolean isFrontProcess = true;
 
     switch (eventPtr->eKind) {
-	case kEventMouseUp:
 	case kEventMouseDown:
+	case kEventMouseUp:
 	case kEventMouseMoved:
 	case kEventMouseDragged:
 	case kEventMouseWheelMoved:
@@ -140,27 +135,27 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 	    return false;
 	    break;
     }
-    status = GetEventParameter(eventPtr->eventRef, 
+    err = ChkErr(GetEventParameter, eventPtr->eventRef,
 	    kEventParamMouseLocation,
-	    typeQDPoint, NULL, 
+	    typeQDPoint, NULL,
 	    sizeof(where), NULL,
 	    &where);
-    if (status != noErr) {
+    if (err != noErr) {
 	GetGlobalMouse(&where);
     }
-    status = GetEventParameter(eventPtr->eventRef, 
+    err = ChkErr(GetEventParameter, eventPtr->eventRef,
 	    kEventParamWindowRef,
-	    typeWindowRef, NULL, 
+	    typeWindowRef, NULL,
 	    sizeof(WindowRef), NULL,
 	    &medPtr->whichWin);
-    if (status == noErr) {
-	status = GetEventParameter(eventPtr->eventRef, 
+    if (err == noErr) {
+	err = ChkErr(GetEventParameter, eventPtr->eventRef,
 		kEventParamWindowPartCode,
-		typeWindowPartCode, NULL, 
+		typeWindowPartCode, NULL,
 		sizeof(WindowPartCode), NULL,
 		&medPtr->windowPart);
     }
-    if (status != noErr) {
+    if (err != noErr) {
 	medPtr->windowPart = FindWindow(where, &medPtr->whichWin);
     }
     medPtr->window = TkMacOSXGetXWindow(medPtr->whichWin);
@@ -169,21 +164,23 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
     }
     if (eventPtr->eKind == kEventMouseDown) {
 	if (IsWindowPathSelectEvent(medPtr->whichWin, eventPtr->eventRef)) {
-	    status = WindowPathSelect(medPtr->whichWin, NULL, NULL);
+	    ChkErr(WindowPathSelect, medPtr->whichWin, NULL, NULL);
 	    return false;
 	}
 	if (medPtr->windowPart == inProxyIcon) {
-	    status = TrackWindowProxyDrag(medPtr->whichWin, where);
-	    if (status == errUserWantsToDragWindow) {
+	    TkMacOSXTrackingLoop(1);
+	    err = ChkErr(TrackWindowProxyDrag, medPtr->whichWin, where);
+	    TkMacOSXTrackingLoop(0);
+	    if (err == errUserWantsToDragWindow) {
 		medPtr->windowPart = inDrag;
 	    } else {
 		return false;
 	    }
 	}
     }
-    status = GetFrontProcess(&frontPsn);
-    if (status == noErr) {
-	SameProcess(&frontPsn, &ourPsn, &isFrontProcess);
+    err = ChkErr(GetFrontProcess, &frontPsn);
+    if (err == noErr) {
+	ChkErr(SameProcess, &frontPsn, &ourPsn, &isFrontProcess);
     }
     if (isFrontProcess) {
 	medPtr->state = ButtonModifiers2State(GetCurrentEventButtonState(),
@@ -193,89 +190,80 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 		GetCurrentKeyModifiers());
     }
     medPtr->global = where;
-    status = GetEventParameter(eventPtr->eventRef, 
+    err = ChkErr(GetEventParameter, eventPtr->eventRef,
 	    kEventParamWindowMouseLocation,
-	    typeQDPoint, NULL, 
+	    typeQDPoint, NULL,
 	    sizeof(Point), NULL,
 	    &medPtr->local);
-    if (status == noErr) {
+    if (err == noErr) {
 	if (medPtr->whichWin) {
 	    Rect widths;
 	    GetWindowStructureWidths(medPtr->whichWin, &widths);
 	    medPtr->local.h -=	widths.left;
-	    medPtr->local.v -=	widths.top;	   
+	    medPtr->local.v -=	widths.top;
 	}
     } else {
 	medPtr->local = where;
 	if (medPtr->whichWin) {
 	    QDGlobalToLocalPoint(GetWindowPort(medPtr->whichWin),
-	            &medPtr->local);
+		    &medPtr->local);
 	}
     }
     medPtr->activeNonFloating = ActiveNonFloatingWindow();
-
-    /*
-     * The window manager only needs to know about mouse down events
-     * and sometimes we need to "eat" the mouse up.  Otherwise, we
-     * just pass the event to Tk.
-
-     */
-    if (eventPtr->eKind == kEventMouseUp) {
-	if (TkMacOSXGetEatButtonUp()) {
-	    TkMacOSXSetEatButtonUp(false);
-	    return false;
-	}
-	return GenerateButtonEvent(medPtr);
-    }
-    if (eventPtr->eKind == kEventMouseDown) {
-	TkMacOSXSetEatButtonUp(false);
-    }
-    if (eventPtr->eKind == kEventMouseWheelMoved) {
-	status = GetEventParameter(eventPtr->eventRef,
-		kEventParamMouseWheelDelta, typeLongInteger, NULL,
-		sizeof(long), NULL, &medPtr->delta);
-	if (status != noErr ) {
-#ifdef TK_MAC_DEBUG
-	    fprintf (stderr,
-		"Failed to retrieve mouse wheel delta, %d\n", (int) status);
-#endif
-	    statusPtr->err = 1;
-	    return false;
-	} else {
-	    EventMouseWheelAxis axis;
-	    status = GetEventParameter(eventPtr->eventRef,
-		    kEventParamMouseWheelAxis, typeMouseWheelAxis, NULL,
-		    sizeof(EventMouseWheelAxis), NULL, &axis);
-	    if (status == noErr && axis == kEventMouseWheelAxisX) {
-		 medPtr->state |= ShiftMask;
-	    }
-	}
-    }
-
     dispPtr = TkGetDisplayList();
     tkwin = Tk_IdToWindow(dispPtr->display, medPtr->window);
 
     if (eventPtr->eKind != kEventMouseDown) {
-	/*
-	 * MouseMoved, MouseDragged or MouseWheelMoved 
-	 */
+	int res = false;
 
-	if (eventPtr->eKind == kEventMouseWheelMoved) {
-	    int res = GenerateMouseWheelEvent(medPtr);
-	    if (res) {
-		 statusPtr->stopProcessing = 1;
-	    }
-	    return res;
-	} else {
-	    return GeneratePollingEvents(medPtr);
+	switch (eventPtr->eKind) {
+	    case kEventMouseUp:
+		/*
+		 * The window manager only needs to know about mouse down
+		 * events and sometimes we need to "eat" the mouse up.
+		 * Otherwise, we just pass the event to Tk.
+		 */
+		if (TkMacOSXGetEatButtonUp()) {
+		    TkMacOSXSetEatButtonUp(false);
+		} else {
+		    res = GenerateButtonEvent(medPtr);
+		}
+		break;
+	    case kEventMouseWheelMoved:
+		err = ChkErr(GetEventParameter, eventPtr->eventRef,
+			kEventParamMouseWheelDelta, typeLongInteger, NULL,
+			sizeof(long), NULL, &medPtr->delta);
+		if (err != noErr ) {
+		    statusPtr->err = 1;
+		} else {
+		    EventMouseWheelAxis axis;
+		    err = ChkErr(GetEventParameter, eventPtr->eventRef,
+			    kEventParamMouseWheelAxis, typeMouseWheelAxis,
+			    NULL, sizeof(EventMouseWheelAxis), NULL, &axis);
+		    if (err == noErr && axis == kEventMouseWheelAxisX) {
+			 medPtr->state |= ShiftMask;
+		    }
+		    res = GenerateMouseWheelEvent(medPtr);
+		}
+		break;
+	    case kEventMouseMoved:
+	    case kEventMouseDragged:
+		res = GeneratePollingEvents(medPtr);
+		break;
+	    default:
+		Tcl_Panic("Unknown mouse event !");
 	}
+	if (res) {
+		statusPtr->stopProcessing = 1;
+	}
+	return res;
     }
-
+    TkMacOSXSetEatButtonUp(false);
     if (medPtr->whichWin) {
 	/*
 	 * We got a mouse down in a window
 	 * See if this is the activate click
-	 * This click moves the window forward.	 We don't want
+	 * This click moves the window forward. We don't want
 	 * the corresponding mouse-up to be reported to the application
 	 * or else it will mess up some Tk scripts.
 	 */
@@ -284,20 +272,20 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 		&& (medPtr->whichWin != medPtr->activeNonFloating
 		|| !isFrontProcess)) {
 	    Tk_Window grabWin = TkMacOSXGetCapture();
-	    if ((grabWin == NULL)) {
+	    if (!grabWin) {
 		int grabState = TkGrabState((TkWindow*)tkwin);
 		if (grabState != TK_GRAB_NONE && grabState != TK_GRAB_IN_TREE) {
 		    /* Now we want to set the focus to the local grabWin */
 		    TkMacOSXSetEatButtonUp(true);
 		    grabWin = (Tk_Window) (((TkWindow*)tkwin)->dispPtr->grabWinPtr);
 		    BringWindowForward(GetWindowFromPort(
-		            TkMacOSXGetDrawablePort(((TkWindow*)grabWin)->window)),
-		            isFrontProcess);
+			    TkMacOSXGetDrawablePort(((TkWindow*)grabWin)->window)),
+			    isFrontProcess);
 		    statusPtr->stopProcessing = 1;
 		    return false;
 		}
 	    }
-	    if ((grabWin != NULL) && (grabWin != tkwin)) {
+	    if (grabWin && grabWin != tkwin) {
 		TkWindow * tkw, * grb;
 		tkw = (TkWindow *)tkwin;
 		grb = (TkWindow *)grabWin;
@@ -314,57 +302,69 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 	     * Clicks in the titlebar widgets are handled without bringing the
 	     * window forward.
 	     */
-            if ((result = HandleWindowTitlebarMouseDown(medPtr, tkwin)) != -1) {
-                return result;
-            } else {
+	    if ((result = HandleWindowTitlebarMouseDown(medPtr, tkwin)) != -1) {
+		return result;
+	    } else {
 		/*
-		 * Allow background window dragging & growing with Command down
+		 * Only windows with the kWindowNoActivatesAttribute can
+		 * receive mouse events in the background.
 		 */
-                if (!((medPtr->windowPart == inDrag ||
-			medPtr->windowPart == inGrow) &&
-			medPtr->state & Mod1Mask)) {
-		    TkMacOSXSetEatButtonUp(true);
-		    BringWindowForward(medPtr->whichWin, isFrontProcess);
-                }
-                /*
-                 * Allow dragging & growing of windows that were/are in the
-                 * background.
-                 */
-                if (!(medPtr->windowPart == inDrag ||
-			medPtr->windowPart == inGrow)) {
-		    return false;
-                }
-            }
+		if (!(((TkWindow *)tkwin)->wmInfoPtr->attributes &
+			kWindowNoActivatesAttribute)) {
+		    /*
+		     * Allow background window dragging & growing with Command
+		     * down.
+		     */
+		    if (!((medPtr->windowPart == inDrag ||
+			    medPtr->windowPart == inGrow) &&
+			    medPtr->state & Mod1Mask)) {
+			TkMacOSXSetEatButtonUp(true);
+			BringWindowForward(medPtr->whichWin, isFrontProcess);
+		    }
+		    /*
+		     * Allow dragging & growing of windows that were/are in the
+		     * background.
+		     */
+		    if (!(medPtr->windowPart == inDrag ||
+			    medPtr->windowPart == inGrow)) {
+			return false;
+		    }
+		}
+	    }
 	} else {
 	    if ((result = HandleWindowTitlebarMouseDown(medPtr, tkwin)) != -1) {
 		return result;
 	    }
 	}
 	switch (medPtr->windowPart) {
-	    case inDrag:
-		SetPort(GetWindowPort(medPtr->whichWin));
-		DragWindow(medPtr->whichWin, where, NULL);
-		where2.h = where2.v = 0;
-		LocalToGlobal(&where2);
-		if (EqualPt(where, where2)) {
-		    return false;
+	    case inDrag: {
+		WindowAttributes attributes;
+
+		GetWindowAttributes(medPtr->whichWin, &attributes);
+		if (!(attributes & kWindowAsyncDragAttribute)) {
+		    TkMacOSXTrackingLoop(1);
+		    DragWindow(medPtr->whichWin, where, NULL);
+		    TkMacOSXTrackingLoop(0);
+		    where2.h = where2.v = 0;
+		    QDLocalToGlobalPoint(GetWindowPort(medPtr->whichWin),
+			    &where2);
+		    if (EqualPt(where, where2)) {
+			return false;
+		    }
+		    return true;
 		}
-		TkMacOSXWindowOffset(medPtr->whichWin, &xOffset, &yOffset);
-		where2.h -= xOffset;
-		where2.v -= yOffset;
-		TkGenWMConfigureEvent(tkwin, where2.h, where2.v,
-			-1, -1, TK_LOCATION_CHANGED);
-		return true;
 		break;
+	    }
 	    case inGrow:
 		/*
 		 * Generally the content region is the domain of Tk
-		 * sub-windows.  However, one exception is the grow	  
-		 * region.	A button down in this area will be handled
-		 * by the window manager.  Note: this means that Tk
+		 * sub-windows. However, one exception is the grow
+		 * region. A button down in this area will be handled
+		 * by the window manager. Note: this means that Tk
 		 * may not get button down events in this area!
 		 */
 		if (TkMacOSXGrowToplevel(medPtr->whichWin, where) == true) {
+		    statusPtr->stopProcessing = 1;
 		    return true;
 		} else {
 		    return GenerateButtonEvent(medPtr);
@@ -386,14 +386,14 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
  *
  * HandleWindowTitlebarMouseDown --
  *
- *	  Handle clicks in window titlebar.
+ *	Handle clicks in window titlebar.
  *
  * Results:
- *	  1 if event was handled, 0 if event was not handled,
- *	  -1 if MouseDown was not in window titlebar. 
+ *	1 if event was handled, 0 if event was not handled,
+ *	-1 if MouseDown was not in window titlebar.
  *
  * Side effects:
- *	  Additional events may be place on the Tk event queue.
+ *	Additional events may be place on the Tk event queue.
  *
  *----------------------------------------------------------------------
  */
@@ -402,40 +402,41 @@ int
 HandleWindowTitlebarMouseDown(MouseEventData * medPtr, Tk_Window tkwin)
 {
     int result = 0;
-    
-    switch (medPtr->windowPart) {
-        case inGoAway:
-            if (TrackGoAway(medPtr->whichWin, medPtr->global)) {
-                if (tkwin) {
-                    TkGenWMDestroyEvent(tkwin);
-                    result = 1;
-                }
-            }
-            break;
-        case inCollapseBox:
-            if (TrackBox(medPtr->whichWin, medPtr->global, medPtr->windowPart)) {
-                if (tkwin) {
-                    TkpWmSetState((TkWindow *)tkwin, IconicState);
-                    result = 1;
-                }
-            }
-            break;
-        case inZoomIn:
-        case inZoomOut:
-            if (TrackBox(medPtr->whichWin, medPtr->global, medPtr->windowPart)) {
-                result = TkMacOSXZoomToplevel(medPtr->whichWin, medPtr->windowPart);
-            }
-            break;
-        case inToolbarButton:
-            if (TrackBox(medPtr->whichWin, medPtr->global, medPtr->windowPart)) {
-                result = GenerateToolbarButtonEvent(medPtr);
-            }
-            break;
-        default:
-            result = -1;
-            break;
-    }
 
+    TkMacOSXTrackingLoop(1);
+    switch (medPtr->windowPart) {
+	case inGoAway:
+	    if (TrackGoAway(medPtr->whichWin, medPtr->global)) {
+		if (tkwin) {
+		    TkGenWMDestroyEvent(tkwin);
+		    result = 1;
+		}
+	    }
+	    break;
+	case inCollapseBox:
+	    if (TrackBox(medPtr->whichWin, medPtr->global, medPtr->windowPart)) {
+		if (tkwin) {
+		    TkpWmSetState((TkWindow *)tkwin, IconicState);
+		    result = 1;
+		}
+	    }
+	    break;
+	case inZoomIn:
+	case inZoomOut:
+	    if (TrackBox(medPtr->whichWin, medPtr->global, medPtr->windowPart)) {
+		result = TkMacOSXZoomToplevel(medPtr->whichWin, medPtr->windowPart);
+	    }
+	    break;
+	case inToolbarButton:
+	    if (TrackBox(medPtr->whichWin, medPtr->global, medPtr->windowPart)) {
+		result = GenerateToolbarButtonEvent(medPtr);
+	    }
+	    break;
+	default:
+	    result = -1;
+	    break;
+    }
+    TkMacOSXTrackingLoop(0);
     return result;
 }
 
@@ -444,16 +445,16 @@ HandleWindowTitlebarMouseDown(MouseEventData * medPtr, Tk_Window tkwin)
  *
  * GeneratePollingEvents --
  *
- *	  This function polls the mouse position and generates X Motion,
- *	  Enter & Leave events.	 The cursor is also updated at this
- *	  time.
+ *	This function polls the mouse position and generates X Motion,
+ *	Enter & Leave events. The cursor is also updated at this
+ *	time.
  *
  * Results:
- *	  True if event(s) are generated - false otherwise.  
+ *	True if event(s) are generated - false otherwise.
  *
  * Side effects:
- *	  Additional events may be place on the Tk event queue.
- *	  The cursor may be changed.
+ *	Additional events may be place on the Tk event queue.
+ *	The cursor may be changed.
  *
  *----------------------------------------------------------------------
  */
@@ -468,12 +469,12 @@ GeneratePollingEvents(MouseEventData * medPtr)
 
     grabWin = TkMacOSXGetCapture();
 
-    if ((!TkpIsWindowFloating(medPtr->whichWin) 
+    if ((!TkpIsWindowFloating(medPtr->whichWin)
 	    && (medPtr->activeNonFloating != medPtr->whichWin))) {
 	/*
 	 * If the window for this event is not floating, and is not the
 	 * active non-floating window, don't generate polling events.
-	 * We don't send events to backgrounded windows.  So either send
+	 * We don't send events to backgrounded windows. So either send
 	 * it to the grabWin, or NULL if there is no grabWin.
 	 */
 
@@ -481,23 +482,23 @@ GeneratePollingEvents(MouseEventData * medPtr)
     } else {
 	/*
 	 * First check whether the toplevel containing this mouse
-	 * event is the grab window.  If not, then send the event
-	 * to the grab window.	Otherwise, set tkWin to the subwindow  
+	 * event is the grab window. If not, then send the event
+	 * to the grab window. Otherwise, set tkWin to the subwindow
 	 * which most closely contains the mouse event.
 	 */
-       
+
 	dispPtr = TkGetDisplayList();
 	rootwin = Tk_IdToWindow(dispPtr->display, medPtr->window);
-	if ((rootwin == NULL) 
+	if ((rootwin == NULL)
 		|| ((grabWin != NULL) && (rootwin != grabWin))) {
 	    tkwin = grabWin;
 	} else {
-	    tkwin = Tk_TopCoordsToWindow(rootwin, 
-		    medPtr->local.h, medPtr->local.v, 
+	    tkwin = Tk_TopCoordsToWindow(rootwin,
+		    medPtr->local.h, medPtr->local.v,
 		    &local_x, &local_y);
 	}
     }
-    
+
     /*
      * The following call will generate the appropiate X events and
      * adjust any state that Tk must remember.
@@ -505,7 +506,7 @@ GeneratePollingEvents(MouseEventData * medPtr)
 
     Tk_UpdatePointer(tkwin, medPtr->global.h, medPtr->global.v,
 	    medPtr->state);
-    
+
     return true;
 }
 
@@ -514,8 +515,7 @@ GeneratePollingEvents(MouseEventData * medPtr)
  *
  * BringWindowForward --
  *
- *	Bring this background window to the front.  We also set state
- *	so Tk thinks the button is currently up.
+ *	Bring this background window to the front.
  *
  * Results:
  *	None.
@@ -527,22 +527,53 @@ GeneratePollingEvents(MouseEventData * medPtr)
  */
 
 static void
-BringWindowForward(WindowRef wRef, Boolean isFrontProcess)
+BringWindowForward(
+    WindowRef wRef,
+    Boolean isFrontProcess)
 {
     if (!isFrontProcess) {
 	ProcessSerialNumber ourPsn = {0, kCurrentProcess};
-	OSStatus status = SetFrontProcess(&ourPsn);
-	if (status != noErr) {
-#ifdef TK_MAC_DEBUG
-	    fprintf(stderr,"SetFrontProcess failed, %d\n", (int) status);
-#endif
+
+	ChkErr(SetFrontProcess, &ourPsn);
+    }
+
+    if (!TkpIsWindowFloating(wRef)) {
+	if (IsValidWindowPtr(wRef)) {
+	    SelectWindow(wRef);
 	}
     }
-    
-    if (!TkpIsWindowFloating(wRef)) {
-	if (IsValidWindowPtr(wRef))
-	    SelectWindow(wRef);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkMacOSXBringWindowForward --
+ *
+ *	Bring this background window to the front (wrapper around
+ *	BringWindowForward()).
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The window is brought forward.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkMacOSXBringWindowForward(
+    WindowRef wRef)
+{
+    OSStatus err;
+    ProcessSerialNumber frontPsn, ourPsn = {0, kCurrentProcess};
+    Boolean isFrontProcess = true;
+
+    err = ChkErr(GetFrontProcess, &frontPsn);
+    if (err == noErr) {
+	ChkErr(SameProcess, &frontPsn, &ourPsn, &isFrontProcess);
     }
+    BringWindowForward(wRef, isFrontProcess);
 }
 
 /*
@@ -564,12 +595,12 @@ BringWindowForward(WindowRef wRef, Boolean isFrontProcess)
 static int
 GenerateMouseWheelEvent(MouseEventData * medPtr)
 {
-    Tk_Window tkwin, rootwin, grabWin;
+    Tk_Window tkwin, rootwin;
     TkDisplay *dispPtr;
     TkWindow  *winPtr;
-    XEvent     xEvent;
+    XEvent xEvent;
 
-    if ((!TkpIsWindowFloating(medPtr->whichWin) 
+    if ((!TkpIsWindowFloating(medPtr->whichWin)
 	    && (medPtr->activeNonFloating != medPtr->whichWin))) {
 	tkwin = NULL;
     } else {
@@ -578,24 +609,22 @@ GenerateMouseWheelEvent(MouseEventData * medPtr)
 	if (rootwin == NULL) {
 	    tkwin = NULL;
 	} else {
-	    tkwin = Tk_TopCoordsToWindow(rootwin, 
-		    medPtr->local.h, medPtr->local.v, 
+	    tkwin = Tk_TopCoordsToWindow(rootwin,
+		    medPtr->local.h, medPtr->local.v,
 		    &xEvent.xbutton.x, &xEvent.xbutton.y);
 	}
     }
-    
+
     /*
      * The following call will generate the appropiate X events and
      * adjust any state that Tk must remember.
      */
 
-    grabWin = TkMacOSXGetCapture();
-
-    if ((tkwin == NULL) && (grabWin != NULL)) {
-	tkwin = grabWin;
+    if (!tkwin) {
+	tkwin = TkMacOSXGetCapture();
     }
     if (!tkwin) {
-       return false;
+	return false;
     }
     winPtr = (TkWindow *) tkwin;
     xEvent.type = MouseWheelEvent;
@@ -622,12 +651,12 @@ GenerateMouseWheelEvent(MouseEventData * medPtr)
  *	next mouse up event
  *
  * Side effects:
- *	  None.
+ *	None.
  *
  *----------------------------------------------------------------------
  */
-static int
-TkMacOSXGetEatButtonUp()
+int
+TkMacOSXGetEatButtonUp(void)
 {
     return gEatButtonUp;
 }
@@ -638,7 +667,7 @@ TkMacOSXGetEatButtonUp()
  * TkMacOSXSetEatButtonUp --
  *
  * Results:
- *	  None.
+ *	None.
  *
  * Side effects:
  *	Sets the flag indicating if we need to eat the
@@ -646,7 +675,7 @@ TkMacOSXGetEatButtonUp()
  *
  *----------------------------------------------------------------------
  */
-static void
+void
 TkMacOSXSetEatButtonUp(int f)
 {
     gEatButtonUp = f;
@@ -655,42 +684,78 @@ TkMacOSXSetEatButtonUp(int f)
 /*
  *----------------------------------------------------------------------
  *
- * TkMacOSXButtonKeyState --
+ * TkMacOSXKeyModifiers --
  *
- *	  Returns the current state of the button & modifier keys.
+ *	Returns the current state of the modifier keys.
  *
  * Results:
- *	  A bitwise inclusive OR of a subset of the following:
- *	  Button1Mask, ShiftMask, LockMask, ControlMask, Mod?Mask,
- *	  Mod?Mask.
+ *	An OS Modifier state.
  *
  * Side effects:
- *	  None.
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+EventModifiers
+TkMacOSXModifierState(void)
+{
+    UInt32 keyModifiers;
+    Boolean isFrontProcess = false;
+
+    if (GetCurrentEvent()) {
+	ProcessSerialNumber frontPsn, ourPsn = {0, kCurrentProcess};
+	OSStatus err = ChkErr(GetFrontProcess, &frontPsn);
+
+	if (err == noErr) {
+	    ChkErr(SameProcess, &frontPsn, &ourPsn, &isFrontProcess);
+	}
+    }
+    keyModifiers = isFrontProcess ? GetCurrentEventKeyModifiers() :
+	    GetCurrentKeyModifiers();
+
+    return (EventModifiers)(keyModifiers & USHRT_MAX);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkMacOSXButtonKeyState --
+ *
+ *	Returns the current state of the button & modifier keys.
+ *
+ * Results:
+ *	A bitwise inclusive OR of a subset of the following:
+ *	Button1Mask, ShiftMask, LockMask, ControlMask, Mod*Mask.
+ *
+ * Side effects:
+ *	None.
  *
  *----------------------------------------------------------------------
  */
 
 unsigned int
-TkMacOSXButtonKeyState()
+TkMacOSXButtonKeyState(void)
 {
     UInt32 buttonState = 0, keyModifiers;
     Boolean isFrontProcess = false;
-    
+
     if (GetCurrentEvent()) {
 	ProcessSerialNumber frontPsn, ourPsn = {0, kCurrentProcess};
-	OSStatus status = GetFrontProcess(&frontPsn);
-	if (status == noErr) {
-	    SameProcess(&frontPsn, &ourPsn, &isFrontProcess);
+	OSStatus err = ChkErr(GetFrontProcess, &frontPsn);
+
+	if (err == noErr) {
+	    ChkErr(SameProcess, &frontPsn, &ourPsn, &isFrontProcess);
 	}
     }
-    
-    if (!gEatButtonUp) {
+
+    if (!TkMacOSXGetEatButtonUp()) {
 	buttonState = isFrontProcess ? GetCurrentEventButtonState() :
 		GetCurrentButtonState();
     }
     keyModifiers = isFrontProcess ? GetCurrentEventKeyModifiers() :
 	    GetCurrentKeyModifiers();
-    
+
     return ButtonModifiers2State(buttonState, keyModifiers);
 }
 
@@ -715,11 +780,11 @@ static unsigned int
 ButtonModifiers2State(UInt32 buttonState, UInt32 keyModifiers)
 {
     unsigned int state;
-    
+
     /* Tk supports at most 5 buttons */
     state = (buttonState & ((1<<5) - 1)) << 8;
-    
-    if (keyModifiers & alphaLock) { 
+
+    if (keyModifiers & alphaLock) {
 	state |= LockMask;
     }
     if (keyModifiers & shiftKey) {
@@ -734,10 +799,10 @@ ButtonModifiers2State(UInt32 buttonState, UInt32 keyModifiers)
     if (keyModifiers & optionKey) {
 	state |= Mod2Mask;		/* option key */
     }
-    if (keyModifiers & kEventKeyModifierNumLockMask) { 
+    if (keyModifiers & kEventKeyModifierNumLockMask) {
 	state |= Mod3Mask;
     }
-    if (keyModifiers & kEventKeyModifierFnMask) { 
+    if (keyModifiers & kEventKeyModifierFnMask) {
 	state |= Mod4Mask;
     }
 
@@ -749,16 +814,16 @@ ButtonModifiers2State(UInt32 buttonState, UInt32 keyModifiers)
  *
  * XQueryPointer --
  *
- *	  Check the current state of the mouse.	 This is not a complete
- *	  implementation of this function.  It only computes the root
- *	  coordinates and the current mask.
+ *	Check the current state of the mouse. This is not a complete
+ *	implementation of this function. It only computes the root
+ *	coordinates and the current mask.
  *
  * Results:
- *	  Sets root_x_return, root_y_return, and mask_return.  Returns
- *	  true on success.
+ *	Sets root_x_return, root_y_return, and mask_return. Returns
+ *	true on success.
  *
  * Side effects:
- *	  None.
+ *	None.
  *
  *----------------------------------------------------------------------
  */
@@ -775,24 +840,62 @@ XQueryPointer(
     int* win_y_return,
     unsigned int* mask_return)
 {
-    if (root_x_return && root_y_return) {
-	Point where;
-	EventRef ev;
-	OSStatus status;
+    int getGlobal = (root_x_return && root_y_return);
+    int getLocal = (win_x_return && win_y_return);
 
-	if ((ev = GetCurrentEvent())) {
-	    status = GetEventParameter(ev, 
-		    kEventParamMouseLocation,
-		    typeQDPoint, NULL, 
-		    sizeof(where), NULL,
-		    &where);
-	}
-	if (!ev || status != noErr) {
-	    GetGlobalMouse(&where);
-	}
+    if (getGlobal || getLocal) {
+	Point where, local;
+	OSStatus err = noErr;
+	int gotMouseLoc = 0;
+	EventRef ev = GetCurrentEvent();
 
-	*root_x_return = where.h;
-	*root_y_return = where.v;
+	if (ev && getLocal) {
+	    err = ChkErr(GetEventParameter, ev, kEventParamWindowMouseLocation,
+		    typeQDPoint, NULL, sizeof(Point), NULL, &local);
+	    gotMouseLoc = (err == noErr);
+	}
+	if (getGlobal || !gotMouseLoc) {
+	    if (ev) {
+		err = ChkErr(GetEventParameter, ev, kEventParamMouseLocation,
+			typeQDPoint, NULL, sizeof(Point), NULL, &where);
+	    }
+	    if (!ev || err != noErr) {
+		GetGlobalMouse(&where);
+	    }
+	}
+	if (getLocal) {
+	    WindowRef whichWin;
+	    if (ev) {
+		err = ChkErr(GetEventParameter, ev, kEventParamWindowRef,
+			typeWindowRef, NULL, sizeof(WindowRef), NULL,
+			&whichWin);
+	    }
+	    if (!ev || err != noErr) {
+		FindWindow(where, &whichWin);
+	    }
+	    if (gotMouseLoc) {
+		if (whichWin) {
+		    Rect widths;
+
+		    ChkErr(GetWindowStructureWidths, whichWin, &widths);
+		    local.h -= widths.left;
+		    local.v -= widths.top;
+		}
+	    } else {
+		local = where;
+		if (whichWin) {
+		    QDGlobalToLocalPoint(GetWindowPort(whichWin), &local);
+		}
+	    }
+	}
+	if (getGlobal) {
+	    *root_x_return = where.h;
+	    *root_y_return = where.v;
+	}
+	if (getLocal) {
+	    *win_x_return = local.h;
+	    *win_y_return = local.v;
+	}
     }
     if (mask_return) {
 	*mask_return = TkMacOSXButtonKeyState();
@@ -803,18 +906,56 @@ XQueryPointer(
 /*
  *----------------------------------------------------------------------
  *
- * TkGenerateButtonEvent --
+ * TkGenerateButtonEventForXPointer --
  *
- *	  Given a global x & y position and the button key status this 
- *	  procedure generates the appropiate X button event.  It also 
- *	  handles the state changes needed to implement implicit grabs.
+ *	This procedure generates an X button event for the current
+ *	pointer state as reported by XQueryPointer().
  *
  * Results:
- *	  True if event(s) are generated - false otherwise.
+ *	True if event(s) are generated - false otherwise.
  *
  * Side effects:
- *	  Additional events may be place on the Tk event queue.
- *	  Grab state may also change.
+ *	Additional events may be place on the Tk event queue.
+ *	Grab state may also change.
+ *
+ *----------------------------------------------------------------------
+ */
+
+MODULE_SCOPE int
+TkGenerateButtonEventForXPointer(
+    Window window)	  /* X Window containing button event. */
+{
+    MouseEventData med;
+    int global_x, global_y, local_x, local_y;
+
+    bzero(&med, sizeof(MouseEventData));
+    XQueryPointer(NULL, None, NULL, NULL, &global_x, &global_y,
+	    &local_x, &local_y, &med.state);
+    med.global.h = global_x;
+    med.global.v = global_y;
+    med.local.h = local_x;
+    med.local.v = local_y;
+    med.window = window;
+    med.activeNonFloating = ActiveNonFloatingWindow();
+
+    return GenerateButtonEvent(&med);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkGenerateButtonEvent --
+ *
+ *	Given a global x & y position and the button key status this
+ *	procedure generates the appropiate X button event. It also
+ *	handles the state changes needed to implement implicit grabs.
+ *
+ * Results:
+ *	True if event(s) are generated - false otherwise.
+ *
+ * Side effects:
+ *	Additional events may be place on the Tk event queue.
+ *	Grab state may also change.
  *
  *----------------------------------------------------------------------
  */
@@ -827,7 +968,7 @@ TkGenerateButtonEvent(
     unsigned int state)	  /* Button Key state suitable for X event */
 {
     MouseEventData med;
-    
+
     bzero(&med, sizeof(MouseEventData));
     med.state = state;
     med.window = window;
@@ -846,49 +987,50 @@ TkGenerateButtonEvent(
  *
  * GenerateButtonEvent --
  *
- *	  Generate an X button event from a MouseEventData structure.
- *	  Handles the state changes needed to implement implicit grabs.
+ *	Generate an X button event from a MouseEventData structure.
+ *	Handles the state changes needed to implement implicit grabs.
  *
  * Results:
- *	  True if event(s) are generated - false otherwise.
+ *	True if event(s) are generated - false otherwise.
  *
  * Side effects:
- *	  Additional events may be place on the Tk event queue.
- *	  Grab state may also change.
+ *	Additional events may be place on the Tk event queue.
+ *	Grab state may also change.
  *
  *----------------------------------------------------------------------
  */
 
-static int
+static int
 GenerateButtonEvent(MouseEventData * medPtr)
 {
     Tk_Window tkwin;
     int dummy;
     TkDisplay *dispPtr;
 
-    /* 
+#if UNUSED
+    /*
      * ButtonDown events will always occur in the front
-     * window.	ButtonUp events, however, may occur anywhere
-     * on the screen.  ButtonUp events should only be sent
+     * window. ButtonUp events, however, may occur anywhere
+     * on the screen. ButtonUp events should only be sent
      * to Tk if in the front window or during an implicit grab.
      */
-    if (0  
-	    && ((medPtr->activeNonFloating == NULL)
-	    || ((!(TkpIsWindowFloating(medPtr->whichWin)) 
+    if ((medPtr->activeNonFloating == NULL)
+	    || ((!(TkpIsWindowFloating(medPtr->whichWin))
 	    && (medPtr->activeNonFloating != medPtr->whichWin))
-	    && TkMacOSXGetCapture() == NULL))) {
+	    && TkMacOSXGetCapture() == NULL)) {
 	return false;
     }
+#endif
 
     dispPtr = TkGetDisplayList();
     tkwin = Tk_IdToWindow(dispPtr->display, medPtr->window);
 
     if (tkwin != NULL) {
-	tkwin = Tk_TopCoordsToWindow(tkwin, medPtr->local.h, medPtr->local.v, 
+	tkwin = Tk_TopCoordsToWindow(tkwin, medPtr->local.h, medPtr->local.v,
 		&dummy, &dummy);
     }
 
-    Tk_UpdatePointer(tkwin, medPtr->global.h,  medPtr->global.v, medPtr->state);
+    Tk_UpdatePointer(tkwin, medPtr->global.h, medPtr->global.v, medPtr->state);
 
     return true;
 }
@@ -925,10 +1067,11 @@ GenerateToolbarButtonEvent(MouseEventData * medPtr)
 		medPtr->local.h, medPtr->local.v, &event.x, &event.y);
     }
     if (!tkwin) {
-       return true;
+	return true;
     }
-
     winPtr = (TkWindow *)tkwin;
+
+    bzero(&event, sizeof(XVirtualEvent));
     event.type = VirtualEvent;
     event.serial = LastKnownRequestProcessed(winPtr->display);
     event.send_event = false;
@@ -937,13 +1080,12 @@ GenerateToolbarButtonEvent(MouseEventData * medPtr)
     event.root = XRootWindow(winPtr->display, 0);
     event.subwindow = None;
     event.time = TkpGetMS();
-
     event.x_root = medPtr->global.h;
     event.y_root = medPtr->global.v;
     event.state = medPtr->state;
     event.same_screen = true;
     event.name = Tk_GetUid("ToolbarButton");
-    Tk_QueueWindowEvent((XEvent *) &event, TCL_QUEUE_TAIL);
 
+    Tk_QueueWindowEvent((XEvent *) &event, TCL_QUEUE_TAIL);
     return true;
 }
